
(** An abstract class representing any objects on screen
    (other than players and the arena) *)

(** Set of (all) bullets (objects of type Bullet.t). *)
type set;;

(** An abstract class representing any objects on screen
    (other than players and the arena). *)
class virtual t : set -> object

  (** Called when it's time for new frame. *)
  method virtual process : unit

  (** Called when there is a need to draw the object on a context. *)
  method virtual draw : Video.t -> unit

  (** Removes the object from the set in which it is. *)
  method delete : unit

  val players : Player.set
  (** Set of players in the game. *)

  val arena : Arena.t
  (** Arena on which there is this object *)
end;;

(** Create new empty set. *)
val create_set : Arena.t -> Player.set -> set;;

(** Execute a function for every object in the set. *)
val iter : (t -> unit) -> set -> unit;;
