
(** Some constants used in the game *)

(** Pi value. *)
let pi_h = asin 1.;;

(** {6 Constants for crosshair.ml} *)

(** How fast the sight angle moves. *)
let aim_speed = 
  Parser.add_variable "aim_speed" (Parser.float ()) 1.2;;
(** Maximum sight angle (down) *)
let aim_angle_max = pi_h -. 0.4;;
(** Minimum sight angle (up ) *)
let aim_angle_min = -. pi_h;;
(** How far is the crosshair from the player. *)
let crosshair_dist = 30.0;;

(** {6 Constants for hook.ml} *)
(** Force coeffitient. Multiplied by the rope length gives the force. *)
let rope_force = 
  Parser.add_variable "rope_force" (Parser.float ()) 4.8;;
(** Color of the rope. *)
let rope_color = (0, 127, 0);;


(* {6 Constants for player.ml} *)
(** Number of lives *)
let lives = 
  Parser.add_variable "lives" (Parser.int ~min:1 ()) 10;;
(** Level from top of the picture of a player when is the center (from where
    the bullets comes). *)
let player_center_y = 7;;
(** Weight of a player. *)
let weight = 
  Parser.add_variable "weight" (Parser.float ()) 320.;;
(** Resistance of the air. *)
let air_viscosity = 
  Parser.add_variable "air_viscosity" (Parser.float ()) 2.8;;
(** How high we can climb. *)
let climbing_skill = 
  Parser.add_variable "climbing_skill" (Parser.float ()) 3.0;;
(** Move speed. *)
let move_speed = 
  Parser.add_variable "move_speed" (Parser.float ()) 40.;;
(** Horizontal speed when in the air. *)
let flight_steering_speed = 
  Parser.add_variable "flight_steering_speed" (Parser.float ()) 24.;;
(** Resistance when on the ground. *)
let resistance = 
  Parser.add_variable "resistance" (Parser.float ()) 8.;;
(** Initial vertical speed when jumping. *)
let jump_speed = 
  Parser.add_variable "jump_speed" (Parser.float ()) 260.;;
(** Speed of the rope when thrown. *)
let rope_speed = 
  Parser.add_variable "rope_speed" (Parser.float ()) 12.;;
(** Minimal time between consecutive weapon changes. *)
let weapon_ch_time = 0.2;;
(** Minimal time after releasing the change+fire combination from when
    jumping is possible. *)
let no_jump_after_robe = 0.1;;

(** {6 Constants for status.ml} *)
(** Height of a status section (bottom of the screen). *)
let status_height = 100;;
(** Height of a health / ammo bars. *)
let status_bar_height = 10;;
(** X position of things in the status section. *)
let status_margin_x = 10;;
(** Y position of the health bar. *)
let status_health_y = 10;;
(** Color of the health bar. *)
let status_health_color = (0, 255, 0);;
(** Y position of the ammo bar. *)
let status_ammo_y = 30;;
(** Color of the ammo bar. *)
let status_ammo_color = (0, 0, 255);;
(** Y posision of the reloading text. *)
let status_reloading_y = 28;;
(** The reloading text. *)
let status_reloading_text = "Reloading...";;
(** Y posision of the lives text. *)
let status_lives_y = 50;;
(** The lives text. *)
let status_lives_text = "Lives: ";;
(** Y posision of the kills text. *)
let status_kills_y = 70;;
(** The kills text. *)
let status_kills_text = "Kills: ";;
(** Blinking time of the reloading text. *)
let status_blinking_time = 0.25;;

(* drop.ml *)
(** Drop probability (drops per second). *)
let drop_probability = 0.1;;
(** Probability that the drop is a first-aid drop. *)
let health_probability = 0.4;;
(** Minimal health points in a first-aid drop. *)
let min_health_drop = 10;;
(** Maximal health points in a first-aid drop. *)
let max_health_drop = 60;;


(* main.ml *)
(** How much weapons has a player at once. *)
let weapons_at_once = 3;;


