
type drop_type = HealthDrop of int | WeaponDrop of AllWeapons.weapon;;

let img_health = Video.load_image "Data/firstaid.png";;
let img_weapon = Video.load_image "Data/weapondrop.png";;

let desc = { StandardBullet.std_desc with 
  StandardBullet.weight = 1000.;
  StandardBullet.air_viscosity = 10.;
  StandardBullet.initial_vel = 0.;
  StandardBullet.life_time = 30.; 
  StandardBullet.bounce_res = 1.; 
};;

let font_white = Font.load "Data/fixed12s.png"(*font.png*);;

let to_int (x, y) = (int_of_float x, int_of_float y);;

(********************** THE DROP CLASS *********************)

class drop arena bullets dtype = 
  let params = {
    Weapon.player = None;
    Weapon.pos = 0, 0; (* don't care *)
    Weapon.vel_mod = 0., 0.; (* don't care *)
    Weapon.angle = 0.; (* don't care *)
    Weapon.dir = Types.Left; (* don't care *)
  } in
  let desc = match dtype with 
      HealthDrop _ -> { desc with StandardBullet.img = img_health }
    | WeaponDrop _ -> { desc with StandardBullet.img = img_weapon }
  in
  object (self)

    inherit StandardBullet.t bullets desc params as super
    
    initializer
      let can_go sprite pos =
        Sprite.move_to sprite pos;
        Sprite.in_bounds sprite && 
          Sprite.collides_with sprite true (Arena.get_sprite_set arena) = []
      in
      let _, _, x1, y1 = Arena.get_boundary arena in
      let rec set_random_pos () = 
        let x, y = Random.int x1, Random.int y1 in
        if can_go sprite (x, y) then
          pos <- float_of_int x, float_of_int y
        else
          set_random_pos ()
      in set_random_pos ()

    method player_collision = function 
        [] -> assert false
      | player :: _ -> begin match dtype with
              HealthDrop health -> Player.injure player (- health) None
            | WeaponDrop (_, weapon) -> Player.add_weapon player (weapon bullets)
          end;
          self#delete
    
    method rock_collision = self#bounce
    method end_of_life = self#explode 25
    method get_frame = 0
    
    method draw cntx = 
      super#draw cntx;
      match dtype with
          HealthDrop _ -> ()
        | WeaponDrop (name, _) -> 
            let width = Font.width font_white name in
            let x, y = to_int pos in
            Font.draw_string ~on:cntx font_white (x - width / 2) (y - 30) name 
    
  end;;

(****************** CREATING NEW DROPS *********************)

let process arena bullets = 
  if Random.float 1. < !Arena.delay *. Constants.drop_probability then
    let typ = if Random.float 1. < Constants.health_probability then
        HealthDrop (Constants.min_health_drop + 
          Random.int (Constants.max_health_drop - Constants.min_health_drop))
      else
        WeaponDrop (AllWeapons.get_random ())
    in
    ignore (new drop arena bullets typ)
;;
