
type state = {
  v : Types.vel;
  hooked : bool;
  released : bool;
}

type t = state Sprite.t * Arena.t;;

let hook_img = Video.load_image "Data/pixel.png";;

let hide (sprite, _) =
  Sprite.set_data sprite {released = false; hooked = false; v = (0., 0.); }
;;
    
let release (sprite, _) pos speed = 
  Sprite.move_to sprite pos;
  Sprite.set_data sprite { v = speed; hooked = false; released = true; }
;;

let create arena =
  let hook_set = Sprite.Set.create 1 in
  Arena.set_boundary arena hook_set;
  let initial_state = {
    v = (0., 0.); 
    hooked = false;
    released = false;
  } in
  Sprite.create ~set:hook_set initial_state hook_img (0, 0), arena
;;

let is_released (sprite, _) = (Sprite.get_data sprite).released;;
let is_hooked (sprite, _) = (Sprite.get_data sprite).hooked;;

let set_hooked (sprite, _) = 
  let state = Sprite.get_data sprite in
  Sprite.set_data sprite {state with hooked = true}
;;

let draw context (sprite, _ as hook) player_pos =
  let x2, y2 = Sprite.get_pos sprite
  and x1, y1 = player_pos in
  if is_released hook then begin
    Video.line ~on:context (x1 + 1, y1) (x2 + 1, y2) Constants.rope_color;
    Video.line ~on:context (x1, y1) (x2, y2) Constants.rope_color;
    Sprite.draw ~on:context sprite
  end
;;

let rope_force (sprite, _ as hook) (px, py) =
  if not (is_hooked hook) then (0., 0.) else
  let hx, hy = Sprite.get_pos sprite in
  let hx, hy = float_of_int (hx - px), float_of_int (hy - py) in
  hx *. !Constants.rope_force, hy *. !Constants.rope_force
;;
    
let process (sprite, arena as hook) =
  if not (is_released hook) || is_hooked hook then () else
  let vx, vy = (Sprite.get_data sprite).v in
  let nx, ny = int_of_float (!Arena.delay *. vx), int_of_float (!Arena.delay *. vy) in
  let chk hook = 
    Sprite.in_bounds hook && Sprite.collides_with hook true (Arena.get_sprite_set arena) = [] 
  in
  let moved = Sprite.move_in_bounds sprite ~chk (nx, ny) in
  if moved <> (nx, ny) then set_hooked hook
;;

