
(** The main module *)

(**/**)

let contexts = ref [];;

(* Create a context and a stat context for each player *)
let video_change_fun () =
  let vertical_split () =
    let res_x, res_y = Video.get_resolution () in
    let local_players = List.length (Net.get_local_players ()) in
    if local_players = 0 || res_x < 3 then [] else
    let cnt_w = res_x / local_players - 2 in
    let rec set_context acc i = 
      if i = local_players then acc else
      let cnt_x = (cnt_w + 2) * i + 1 in
      let context = Video.create_context cnt_x 0 cnt_w (res_y - Constants.status_height) (0, 0) 
      and stat_context = 
        Video.create_context cnt_x (res_y - Constants.status_height) cnt_w 
          Constants.status_height (0,0) 
      in
      set_context ((context, stat_context, i) :: acc) (i + 1)
    in
    set_context [] 0
  in
  contexts := vertical_split ()
;;

Helpers.add_video_mode_var video_change_fun;;
Parser.parse "exec \"Data/default.txt\"" ();;

let arena = Arena.create ();;
let players = Player.create_set arena;;
let bullets = Bullet.create_set arena players;;

let create_random_weapons () = 
  List.map (fun (_, x) -> x bullets) (AllWeapons.get_random_list Constants.weapons_at_once)
;;

(* new player / delete player *)
let pl_change_fun pl_no local = function
  | true -> 
      Player.create players pl_no (create_random_weapons ());
      if local then video_change_fun ()
  | false -> 
      Player.delete (Player.by_num players pl_no); 
      if local then video_change_fun ()
;;

Net.init pl_change_fun;;
Helpers.add_chat ();;

Parser.parse "exec \"Data/liero.txt\"" ();;

let draw () =
  Video.fill (0, 0, 0);
  let draw_context (context, stat_context, pl_no) =
    let pl = Player.by_num players pl_no in
    Status.draw pl stat_context;
    Arena.set_cam_pos_center arena context (Player.get_pos pl);
    Arena.draw arena context;
    Player.iter (fun p -> Player.draw p context) players;
    Bullet.iter (fun b -> b#draw context) bullets;
  in List.iter draw_context !contexts
;;

let process () =
  Arena.delay := 1. /. max 20. !Helpers.fps;
  Drop.process arena bullets;
  Bullet.iter (fun b -> b#process) bullets;
  Status.process ();
  Player.iter Player.process players
;; 

Random.self_init ();;

Helpers.main process draw;;
