
(** Player character *)

(** The player. *)
type t;;

(** Set of (all) players. *)
type set;;

(** Status of a player. *)
type status = {
  health100 : int; (** percentage health of the player *)
  ammo100 : int; (** percentage ammo of the currently choosen weapon *)
  is_reloading : bool; (** is the player during reloading *)
  lives : int; (** lives left *)
  kills : int; (** number of kills *)
};;

(** Parameters of a shoot. *)
type shoot_params = t Weapon.shoot_params;;

(** [create set pl_no weapons] creates a new player with number [pl_no] and
    weapons [weapons] and adds it to the set [set]. *)
val create : set -> int -> t Weapon.t list -> unit;;

(** Call when there is a need to draw the object on a context. Draws the
    player on the context. *)
val draw : t -> Video.t -> unit;;

(** Call when it's time for new frame. Changes the position of a player
    according to all forces influancing it. Processes pressed buttons. Moves
    also the rope and the crosshair. *)
val process : t -> unit;;

(** @return the player position. *)
val get_pos : t -> Types.pos;;

(** @return direction of a player. *)
val get_direction : t -> Types.dir;;

(** @return the looking angle. Lower values means higher. Up is -pi, down is
    +pi. *)
val get_angle : t -> Types.angle;;

(** @return the status of the player. *)
val get_status : t -> status;;

(** [injure player health by_who] decreases [health] life points of player
    [player]. If he dies it's known that [by_who] killed him. When health is
    negative it increases the life points of a player. *)
val injure : t -> int -> t option -> unit;;

(** Adds new weapon to player. The currently selected weapon is replaced. *)
val add_weapon : t -> t Weapon.t -> unit;;

(** Create an empty set of (all) players. *)
val create_set : Arena.t -> set;;

(** @return the player having given number. @raise Not_found when there is
    no such player. *) 
val by_num : set -> int -> t;;

(** Remove given player from the set in which he is. *)
val delete : t -> unit;;

(** Execute a function for all players in the set. *)
val iter : (t -> unit) -> set -> unit;;

(** @return list of players colliding with the given sprite. *)
val collides_with : set -> 'a Sprite.t -> t list;;

