
(** Weapons *)

(** Description of a shoot. *)
type 'player shoot_params = {
  player : 'player option; (** player who shoots *)
  pos : Types.pos; (** position of the player *)
  vel_mod : Types.vel; (** velocity of the player (will be added to initial velocity of the bullet) *)
  angle : Types.angle; (** looking angle *)
  dir : Types.dir; (** looking direction *)
};;

(** The weapon. *)
type 'player t;;

(** A function which will be called when shooting. *)
type 'player fire_fun = 'player shoot_params -> unit;;

(** [create fire_fun amount_of_bullets reloading_time wait_time name]
    creates new weapon with shooting function [fire_fun], number of ammo
    in a magazine [amount_of_bullets], reloading time [reloading_time] and
    time between shoots [wait_time]. Seperate weapons should be created for
    every player. *)
val create : 'player fire_fun -> int -> float -> float -> string -> 'player t;;

(** A function to be called to check if the continouous shooting already ended. *)
type check_stop_fun = unit -> bool;;

(** A function which will be called to start continouous shooting. *)
type 'player start_fire_fun = 'player shoot_params -> check_stop_fun -> unit;;

(** [create_cont fire_fun stop_fun max_time reloading_time name] creates new
    weapon of continuous shooting with start function [fire_fun], time of
    shooting between reloads [max_time] and reloading time [reloading_time].
    Seperate weapons should be created for every player. *)
val create_cont : 'player start_fire_fun -> float -> float -> string -> 'player t;;

(** Called when it's time for new frame. Updates the counters of reloading /
    wait time. *) 
val process : 'player t -> unit;;

(** Shoot. *)
val fire : 'player t -> 'player shoot_params -> unit;;

(** @return percentage ammo available. *)
val get_ammo100 : 'player t -> int;;

(** Are we during reloading the weapon? *)
val is_reloading : 'player t -> bool;;

(** @return the name of the weapon. *)
val get_name : 'player t -> string;;

