
let all = ref [];;

type action = Explode | Bounce | NoCom | Delete | Injure;;

(* type for weapon description readed from file *)
type config = {
  mutable name : string;
  
  (* description for standardBullet *)
  mutable weight : float;
  mutable air_viscosity : float;
  mutable initial_vel : float;
  mutable velocity_vary : float;
  mutable life_time : float;
  mutable bounce_res : float;
  mutable img : string;
  
  (* actions descriptios for standardBullet *)
  mutable explosion_force : int;
  mutable rock_collision : action;
  mutable player_collision : action;
  mutable end_of_life : action;

  (* for creating weapon *)
  mutable bullets : int;
  mutable reloading_time : float;
  mutable wait_time : float;
};;

(* default weapon configuration *)
let default_config = {
  name = "";
  weight = StandardBullet.std_desc.StandardBullet.weight;
  air_viscosity = StandardBullet.std_desc.StandardBullet.air_viscosity;
  initial_vel = StandardBullet.std_desc.StandardBullet.initial_vel;
  velocity_vary = StandardBullet.std_desc.StandardBullet.velocity_vary;
  life_time = StandardBullet.std_desc.StandardBullet.life_time;
  bounce_res = StandardBullet.std_desc.StandardBullet.bounce_res;
  img = "";
  explosion_force = 25;
  rock_collision = Delete;
  player_collision = Injure;
  end_of_life = NoCom;
  bullets = 1;
  reloading_time = 2.;
  wait_time = 0.1;
};;

(* weapon being currenly readed from file *)
let cur = ref default_config;;

let read_action = function 
    "bounce" -> Bounce
  | "explode" -> Explode
  | "delete" -> Delete
  | "injure" -> Injure
  | str -> Log.error ("invalid action" ^ str); NoCom
;;

Parser.add_command "weapon" (Parser.pair Parser.string Parser.string) "Begins weapon definition"
  (fun (n, im) -> 
    cur := { default_config with name = n; img = im}; 
);;

Parser.add_command "weight" (Parser.float ()) "" (fun x -> !cur.weight <- x);;
Parser.add_command "air_viscosity" (Parser.float ()) "" (fun x -> !cur.air_viscosity <- x);;
Parser.add_command "initial_vel" (Parser.float ()) "" (fun x -> !cur.initial_vel <- x);;
Parser.add_command "velocity_vary" (Parser.float ()) "" (fun x -> !cur.velocity_vary <- x);;
Parser.add_command "bounce_res" (Parser.float ()) "" (fun x -> !cur.bounce_res <- x);;
Parser.add_command "life_time" (Parser.float ()) "" (fun x -> !cur.life_time <- x);;
Parser.add_command "bullets" (Parser.int ~min:1 ()) "" (fun x -> !cur.bullets <- x);;
Parser.add_command "reloading_time" (Parser.float ()) "" (fun x -> !cur.reloading_time <- x);;
Parser.add_command "wait_time" (Parser.float ()) "" (fun x -> !cur.wait_time <- x);;
Parser.add_command "explosion_force" (Parser.int ~min:1 ()) "" (fun x -> !cur.explosion_force <- x);;
Parser.add_command "rock_collision" Parser.string "" (fun x -> !cur.rock_collision <- read_action x);;
Parser.add_command "player_collision" Parser.string "" (fun x -> !cur.player_collision <- read_action x);;
Parser.add_command "end_of_life" Parser.string "" (fun x -> !cur.end_of_life <- read_action x);;

class bullet set config img params = 
let desc = {
  StandardBullet.std_desc with
  StandardBullet.weight = config.weight;
  StandardBullet.air_viscosity = config.air_viscosity;
  StandardBullet.initial_vel = config.initial_vel;
  StandardBullet.velocity_vary = config.velocity_vary;
  StandardBullet.life_time = config.life_time;
  StandardBullet.bounce_res = config.bounce_res;
  StandardBullet.img = img;
} in
  object (self)
    inherit StandardBullet.t set desc params
    
    method action ?player_list = function 
        NoCom -> ()
      | Bounce -> self#bounce
      | Delete -> self#delete
      | Explode -> self#explode config.explosion_force
      | Injure -> begin 
          self#delete;
          match player_list with 
              None -> Log.error "injure command not in player_collision"
            | Some player_list -> List.iter (fun pl -> 
                  Player.injure pl config.explosion_force params.Weapon.player
                ) player_list
          end
    
    method rock_collision = self#action config.rock_collision
    method player_collision player_list = self#action ~player_list config.player_collision
    method end_of_life = self#action config.end_of_life
    method get_frame = 0
  end
;;

let fire set config img params = ignore (new bullet set config img params);;

let create config img set = Weapon.create (fire set config img) config.bullets 
  config.reloading_time config.wait_time config.name
;;

Parser.add_command "end_weapon" Parser.unit "Ends weapon definition"
  (fun () -> 
    let img = Video.load_image ("Data/" ^ !cur.img) in
    let weapon = create !cur img in
    all := (!cur.name, weapon) :: !all
  )
;;

Parser.parse "exec \"Data/weapons.txt\"" ();;



